;
;**************************************************************************************************
;DOS/65 C02 (CMOS) RAM Version
;
;This is Version 3.10 (KM 2022)
;This is the bootable image for the RAM loaded version of DOS/65 (CMOS version)
;
; Based on Richard A. Leary's DOS/65 V3 code - many thanks Richard!!
;
; The target platform for this build is:
;  A prototype 3.3-Volt system based on the C02 Pocket SBC - KM
;  This prototype uses a NXP 28L92 DUART and a Hitachi MicroDrive PATA interface
;  The system has been modified for more RAM, now at 56KB RAM and 8KB ROM, 8MHz CPU
;
;**************************************************************************************************
; Assembler directives
;
                PW      132
                PL      66
                CHIP    W65C02S ;Enable WDC 65C02 instructions
                GLOBALS ON
                CASE    OFF
                INCLIST ON
;
;**************************************************************************************************
;
; This version uses many CMOS instructions and addressing modes!
;
; There are multiple modules that make up this bootable version and use "include" to
; add them into the main code. The Code Modules are:
;       DOS65-MAIN-V310.ASM (This file)
;       DOS65-CCM-V310.ASM - Concurrent Command Module
;       DOS65-PEM-V310.ASM - Primitive Execution Module
;       DOS65-SIM-V310.ASM - System Interface Module
;
; Other files required contain system variables and constants for C02BIOS and C02Monitor,
; which are included in the SIM source module.
;
;  The current BIOS and Monitor versions for the Prototype system are 4.02
;
;**************************************************************************************************
;
;               25 October 2022 - KM
;Major revision! This is an entirely RAM based version of DOS/65
;
; Based on my 3.04 CMOS ROM Version
; - All data areas are included in the linked modules
; - Allocation Maps for 8 drives are zeroed and add 4KB to the 6KB of DOS/65 code
; - Supports 8- 8MB drives configured on the Microdrive.
;
; ALL modules (CCM, PEM, SIM) have been reworked for RAM base and include data areas.
;
;**************************************************************************************************
;
;Base addresses and definitions
pzstrt          equ     $88             ;start of page zero system RAM
btejmp          equ     $100            ;warm boot jump
pemjmp          equ     $103            ;jump to PEM
iostat          equ     $106            ;I/O status
dflfcb          equ     $107            ;default FCB
dflbuf          equ     $128            ;default buffer
tea             equ     $800            ;TEA start - KM
;
;C02 Pocket SBC memory parameters
; NOTE: The ramstrt should be changed to $B800!
; this provides a TEA area from $0800 to $B7FF, which is 44KB
;
ramstrt         equ     $5800           ;default RAM location initial testing - KM
hstbuf          equ     $0600           ;moved host buffer to low RAM - KM
;
;Fixed parameters
lf              equ     $0A             ;linefeeed
cr              equ     $0D             ;return
eof             equ     $1A             ;end of file
null            equ     0               ;null
ctlc            equ     3               ;abort
ctle            equ     5               ;physical cr lf
ctli            equ     9               ;tab character
ctlp            equ     $10             ;toggle printer
ctlr            equ     $12             ;repeat line
ctls            equ     $13             ;freeze
ctlx            equ     $18             ;cancel
semico          equ     $3B             ;semicolon
delete          equ     $7F             ;delete character
numcmd          equ     37              ;number of PEM commands
;
;Page Zero use
                page0
;CCM / PEM
addinp          equ     pzstrt          ;initialized to a,y
bufadd          equ     addinp+2        ;buffer address
alcpnt          equ     bufadd+2        ;allocation map pointer
chkpnt          equ     alcpnt+2        ;checksum map pointer
msgptr          equ     chkpnt+2        ;message pointer
movptr          equ     msgptr          ;and move pointer
dcbloc          equ     msgptr+2        ;pointer to dcb
;
;sim
sim_dmaadr      equ     dcbloc+2        ;record buffer
sim_dcbpnt      equ     sim_dmaadr+2    ;pointer into selected dcb
sim_movepnt     equ     sim_dcbpnt+2    ;read or write move host pointer
;
;mon/bios
IDE_LBA0        equ     sim_movepnt+2   ;IDE LBA Variables for DOS/65
IDE_LBA1        equ     sim_movepnt+3
IDE_LBA2        equ     sim_movepnt+4
;
mon_nxtdrv      equ     IDE_LBA2+1      ;SIM access variables
mon_nxttrk      equ     IDE_LBA2+2
mon_nxtsec      equ     IDE_LBA2+3
;
dos65_flg       equ     pzstrt+119      ;borrow spare bits from BIOS MATCH flag
                                        ;bit 0 used for submit flag in CCM
                                        ;bit 4 used for outflg in PEM
                                        ;bit 5 used for drvflg in PEM
;
                ends
;
; Main program code starts here:
        org     ramstrt
;
        include         DOS65-CCM-V310.ASM      ;Include CCM Module
;
        org     $/256*256+256                   ;Align to Page address
;
        include         DOS65-PEM-V310.ASM      ;Include PEM Module
;
        org     $/256*256+256                   ;Align to Page address
;
        include         DOS65-SIM-V310.ASM      ;Include SIM Module
;
        org     $/256*256+256                   ;Align to Page address
;
alloc_maps
;allocation and checksum maps
;drive a
almpa           equ     alloc_maps
        BLKW    256
;drive b
almpb           equ     almpa+512
        BLKW    256
;drive c
almpc           equ     almpb+512
        BLKW    256
;drive d
almpd           equ     almpc+512
        BLKW    256
;drive e
almpe           equ     almpd+512
        BLKW    256
;drive f
almpf           equ     almpe+512
        BLKW    256
;drive g
almpg           equ     almpf+512
        BLKW    256
;drive h
almph           equ     almpg+512
        BLKW    256
;
        end

