;
;DOS/65 system interface module (SIM)
;Version 1.26 - RAM based
;
;This is the WDC tools version derived from SIM415X.ASM.
;This version is designed to work with the:
;C02 Pocket SBC with RTC/CF-Card adapter:
; - modified for 3.3V for MicroDrive and DS1511Y RTC
;
;Microdrive Disk configuration:
; a CF 8MB
; b CF 8MB
; c CF 8MB
; d CF 8MB
; e CF 8MB
; f CF 8MB
; g CF 8MB
; h CF 8MB
;
;Note: This SIM version does not require a DOS/65 Monitor module!
; All required code is part of this SIM version and interfaces directly to C02BIOS/C02Monitor.
; This saves RAM space and eliminates additional JMPs and JSRs.
;
;               2 March 2022 - KM (1.24)
;Minor update to SIM version
; Adds second serial port for punch/reader functions
; Shows updated DOS/65 version now at 3.04
;
;               13th September 2022 - KM (1.25)
;Minor update to SIM version
; Code reduction for block size check, all IDE devices are the same:
; - 4 records per sector
; - 16 records per block
; Increase suppported drive to the maximum of 8 devices at 8MB each.
;
;               25th October 2022 - KM (1.26)
;This version is RAM Based!!
; supports DOS/65 V3.10 bootable RAM based image/
; - can be booted from an IDE drive and jumps to sim_boot address which is fixed to a page
;
;Note that this SIM version uses an additional 4-byte offset for LBA accesses!
; - This is required due to being a bootable system. The Microdrive setup uses a Partition Record
; - which will load the Boot Block from the active partition. The Boot Block and Boot Image are
; - NOT part of the allocated disk space that becomes the drives. This provides an additional
; - layer of safety and isolation. The Boot Block code will use the Load Header at the start of
; - CCM to provide the offset to allocated space and update the Partition Offset variable in
; - this SIM module (part_offset). This design allows the Boot Image to be easily moved to a new
; - location on the drive and not have to worry about the disk access. Only the Boot Block needs
; - to be updated.
;
;**************************************************************************************************
;Include required constants/equates/tables for SIM to assemble
;
        NOLIST
        INCLUDE         C02Constants4.asm       ;Constants/Equates - C02 BIOS/Monitor/Hardware
        INCLUDE         C02JMP_Table.asm        ;Jump Table entries for C02 BIOS/Monitor
        LIST
;
;**************************************************************************************************
;
;**************************************************************************************************
;Fixed parameters
;
CLREOL          .EQU     1              ;clear to eol (ctrl-a)
CLREOS          .EQU     2              ;clear to eos (ctrl-b)
BS              .EQU     8              ;backspace (ctrl-h)
ESC             .EQU     $1B            ;escape character
;
;**************************************************************************************************
;
;PEM constants on entry to write
wrall           .EQU    0               ;write to allocated
wrdir           .EQU    1               ;write to directory
wrual           .EQU    2               ;write to unallocated
;
simram
;
;Main program, Start of SIM
sim
                jmp     sim_boot        ;from cold start        simram+0
sim_wboote      jmp     sim_wboot       ;from warm boot         simram+3
                jmp     mon_consts      ;check for input        simram+6
                jmp     mon_conrde      ;get input              simram+9
                jmp     mon_conwrt      ;send to terminal       simram+12
                nop                     ;printer output         simram+15
                nop
                rts
                jmp     mon_punch       ;punch output           simram+18
                jmp     mon_reader      ;reader input           simram+21
                jmp     sim_home        ;home drive             simram+24
                jmp     sim_seldsk      ;select disk            simram+27
                jmp     sim_seltrk      ;set track              simram+30
                jmp     sim_selsec      ;set sector             simram+33
                jmp     sim_setdma      ;set buffer address     simram+36
                jmp     sim_read        ;read sector            simram+39
                jmp     sim_write       ;write sector           simram+42
                lda     #1              ;printer always ready   simram+45
                rts
                ldx     #128            ;clock entry - disabled simram+48
                rts
                jmp     sim_xlate       ;translate (SIM RTS)    simram+51
;
;console definition block
                byte    0               ;scratch
sysdef          byte    8               ;backspace
                byte    1               ;clear to end of line
                byte    $0C             ;forward space
                byte    0               ;normal video
                byte    '^'             ;invert video
                byte    24              ;lines per screen
                byte    80              ;char per line
                byte    $0C             ;form feed
                byte    $1E             ;home
                byte    2               ;clear to end of screen
;
;Opening id message
sim_opnmsg      .DB     CR,LF,"SIM V1.26"
                .DB     CR,LF,"DOS/65 V3.10",0
;
IO_ERR_MSG      .DB     CR,LF,"Disk I/O Error!",0
;
sim_boot        lda     #<sim_opnmsg    ;point to message
                ldy     #>sim_opnmsg
                jsr     m_promptr       ;send it
;
                ldx     #$FF            ;set stack
                txs                     ;pointer
;
;Initialize iostat
                stz     iostat          ;clear I/O status
;
                jsr     b_ide_reset     ;Reset IDE Controller
;
;Set up jumps into dos/65 in page one
; NOTE: changed to absolute indexed using Y register.
;       WDC Tools has a bug using X register when the
;       operand is $0100-1 (should be $00FF, but becomes $FF)
;
sim_setup
;First clear key de-blocking variables
                stz     sim_hstact      ;set host buffer inactive
                stz     sim_unacnt      ;clear unalloc count
;
                ldy     #6              ;set index to 6
simset_lp       lda     sim_inttbl-1,y  ;get SIM table data
                sta     $0100-1,y       ;store into page $01 PEM area
                dey                     ;decrement index
                bne     simset_lp       ;loop until done
                lda     #<dflbuf        ;get low buffer
                ldy     #>dflbuf        ;and high buffer
                jsr     sim_setdma      ;and set DMA
                jmp     ccmram          ;then go to CCM
;Initialization table
sim_inttbl      .DB     $4C             ;JMP opcode
                .DW     sim_wboote      ;warm boot location
                .DB     $4C             ;JMP opcode
                .DW     pemram          ;PEM RAM location
;
;Warm Boot - read DOS/65 back except SIM and then jump to CCM.
; IOSTAT and hence the default drive/user are not altered by Warm Boot.
; Note: WARM BOOT does not read from any drive as it is ROM based.
; It is only a refresh of ROM data to RAM and sets the default drive.
;
sim_wboot       ldx     #$FF            ;set stack
                txs                     ;pointer
                lda     iostat          ;set default drive
                jsr     sim_seldsk
                bra     sim_setup       ;go set up
;
;Select disk
sim_seldsk      and     #%00000111      ;eight drives only
                sta     mon_nxtdrv      ;Save active Drive number
                sta     sim_sekdsk      ;save for later
                asl     a               ;multiply by two
                tax                     ;make an index
                lda     sim_dcbtbl,x    ;get address
                ldy     sim_dcbtbl+1,x
                rts
;Table of dcb addresses.
; - All eight drives are in use for this new SIM version (1.25) - KM
; added a "_" in dcb definitions, as dcbh causes assembly error - KM
sim_dcbtbl      .DW     dcb_a
                .DW     dcb_b
                .DW     dcb_c
                .DW     dcb_d
                .DW     dcb_e
                .DW     dcb_f
                .DW     dcb_g
                .DW     dcb_h
;
;NOTE:
; de-blocking is always used for IDE devices, as the default
; sector size is 512 bytes. All drives are configured with
; the same defaults of:
; 4 records per sector
; 16 records per block
;
; This results in the following routines no longer needed,
; as the disk access routines default to de-blocking for all
; supported drives.
;
;*************************************************************************************************
;See if de-blocking required for sekdsk
; returns number of DOS/65 records per physical sector in A and
; returns Z=1 if no de-blocking required else Z=0
;sim_tstdbl      ldx     sim_sekdsk      ;get desired disk
;
;See if de-blocking required for disk x
;sim_tstdbx      lda     sim_spttbl,x    ;get DOS/65 records/host sector
;                cmp     #1              ;test for no deblock
;                rts                     ;return
;
;Table containing number of DOS/65 records per host physical sector.
; if entry is 1 then de-blocking is skipped.
; note that these two tables contain 8 entries for 8 drives.
;sim_spttbl      .DB     4,4,4,4,4,4,4,4
;
;Table of records/block
;sim_rbltbl      .DB     16,16,16,16,16,16,16,16
;
;Translate record is no longer needed for the Microdrive.
; A borrowed RTS (sim_xlate) is used below for the routine,
; which is still in the PEM code module for compatibility.
;
;*************************************************************************************************
;Home the selected disk
sim_home        lda     sim_hstwrt      ;check for pending write
                bne     sim_xlate       ;there is so skip
                stz     sim_hstact      ;else, clear host active flag
sim_xlate       rts                     ;return
;
;Read the selected DOS/65 record.
sim_read        stz     sim_unacnt      ;clear unallocated count
                ldx     #1              ;x <-- 1
                stx     sim_readop      ;say is read operation
                stx     sim_rsflag      ;must read data
                inx                     ;x <-- wrual
                stx     sim_wrtype      ;treat as unalloc
                jmp     sim_rwoper      ;to perform the read
;
;Write the selected DOS/65 record.
sim_write       sta     sim_wrtype      ;save param from pem
                stz     sim_readop      ;say is not a read operation
                lda     sim_wrtype      ;get write type back
                cmp     #wrual          ;write unallocated?
                bne     ?B3             ;check for unalloc
;
;Write to unallocated, set parameters
                ldx     sim_sekdsk      ;get next disk number
                lda     #16             ;IDE always 16 records/block
                sta     sim_unacnt
                stx     sim_unadsk      ;unadsk <-- sekdsk
                lda     sim_sektrk
                ldy     sim_sektrk+1
                sta     sim_unatrk      ;unatrk <-- sectrk
                sty     sim_unatrk+1
                lda     sim_seksec
                ldy     sim_seksec+1
                sta     sim_unasec      ;unasec <-- seksec
                sty     sim_unasec+1
;
;Check for write to unallocated record
?B3             lda     sim_unacnt      ;any unalloc remain?
                beq     sim_alloc       ;skip if not
;
;More unallocated records remain
                dec     sim_unacnt      ;unacnt <-- unacnt-1
                lda     sim_sekdsk
                cmp     sim_unadsk      ;sekdsk = unadsk?
                bne     sim_alloc       ;skip if not
;
;Disks are the same
                lda     sim_unatrk      ;sektrk = unatrk?
                cmp     sim_sektrk
                bne     sim_alloc       ;no so skip
                lda     sim_unatrk+1
                cmp     sim_sektrk+1
                bne     sim_alloc       ;skip if not
;
;Tracks are the same
                lda     sim_unasec      ;seksec = unasec?
                cmp     sim_seksec
                bne     sim_alloc       ;no so skip
                lda     sim_unasec+1
                cmp     sim_seksec+1
                bne     sim_alloc       ;skip if not
;
;Match, move to next sector for future ref
                inc     sim_unasec      ;unasec = unasec+1
                bne     ?B4
                inc     sim_unasec+1
;
;Calculate DOS/65 records/track
?B4             lda     sim_sekdsk      ;get disk number
                asl     a               ;mult by two
                tax                     ;make an index
                lda     sim_dcbtbl,x    ;get dcb start
                ldy     sim_dcbtbl+1,x
                sta     sim_dcbpnt      ;set low operand
                sty     sim_dcbpnt+1    ;then high operand
;
;Point has address now get spt at byte 2,3
                ldy     #2              ;start at byte 2
                ldx     #0              ;start save in low
?B5             lda     (sim_dcbpnt),y  ;get value
                sta     sim_d65spt,x    ;and save
                iny
                inx
                cpx     #2              ;see if done
                bne     ?B5             ;loop if not
;
;Check for end of track
                lda     sim_unasec      ;end of track?
                cmp     sim_d65spt      ;count dos/65 sectors
                lda     sim_unasec+1
                sbc     sim_d65spt+1
                bcc     ?B6             ;skip if no overflow
;
;Overflow to next track
                stz     sim_unasec      ;unasec <-- 0
                stz     sim_unasec+1
                inc     sim_unatrk      ;unatrk <-- unatrk+1
                bne     ?B6
                inc     sim_unatrk+1
;
;Match found, mark as unnecessary read
?B6             stz     sim_rsflag      ;rsflag <-- 0
                bra     sim_rwoper      ;to perform the write
;
;Not an unallocated record, requires pre-read
sim_alloc       stz     sim_unacnt      ;unacnt <-- 0
;
;Say preread required
                ldx     #1              ;x <-- 1
                stx     sim_rsflag      ;rsflag <-- 1
;
;Common code for read and write follows enter here to perform the read/write
sim_rwoper      stz     sim_erflag      ;set no errors (yet)
                lda     sim_seksec      ;compute host sector
                ldy     sim_seksec+1
                sta     sim_sekhst
                sty     sim_sekhst+1
                lda     #2              ;IDE always 4 records/sector (divided by 2 here)
                tax                     ;make a counter
                beq     ?B2             ;done if zero
?L2             lsr     sim_sekhst+1    ;do high
                ror     sim_sekhst      ;then low
                dex
                bne     ?L2             ;loop if more
;
;Active host sector?
?B2             lda     sim_hstact      ;host active flag
                pha                     ;save flag
                inx                     ;x <-- 1
                stx     sim_hstact
                pla                     ;get flag back
                beq     sim_filhst      ;fill host if not active
;
;Host buffer active, same as seek buffer?
                lda     sim_sekdsk
                cmp     sim_hstdsk      ;same disk?
                bne     sim_nmatch
;
;Same disk, same track?
                lda     sim_hsttrk      ;sektrk = hsttrk?
                cmp     sim_sektrk
                bne     sim_nmatch      ;no
                lda     sim_hsttrk+1
                cmp     sim_sektrk+1
                bne     sim_nmatch
;
;Same disk, same track, same sector?
                lda     sim_sekhst      ;sekhst = hstsec?
                cmp     sim_hstsec
                bne     sim_nmatch      ;no
                lda     sim_sekhst+1
                cmp     sim_hstsec+1
                beq     sim_match       ;skip if match
;
;Proper disk, but not correct sector
sim_nmatch      lda     sim_hstwrt      ;host written?
                beq     sim_filhst      ;skip if was
                jsr     sim_writeh      ;else clear host buff
;
;May have to fill the host buffer so set host parameters
sim_filhst      lda     sim_sekdsk
                sta     sim_hstdsk
                lda     sim_sektrk
                ldy     sim_sektrk+1
                sta     sim_hsttrk
                sty     sim_hsttrk+1
                lda     sim_sekhst
                ldy     sim_sekhst+1
                sta     sim_hstsec
                sty     sim_hstsec+1
                lda     sim_rsflag      ;need to read?
                beq     sim_noread      ;no
;
;Read desired physical sector from host
                jsr     sim_hcom        ;set parameters
                jsr     mon_rdesec      ;to rom
                sta     sim_erflag      ;save result
sim_noread      stz     sim_hstwrt      ;clear, no pending write
;
;Copy data to or from buffer
sim_match       stz     sim_movepnt     ;clear pointer, later we'll set read
                stz     sim_movepnt+1
                lda     #4              ;IDE always 4 records/sector
                dec     a               ;decrement by one
                and     sim_seksec      ;mask sector number
                tax                     ;make a counter
                beq     sim_nooff       ;done if zero
?L2             clc
                lda     sim_movepnt
                adc     #128
                sta     sim_movepnt
                lda     sim_movepnt+1
                adc     #0
                sta     sim_movepnt+1
                dex
                bne     ?L2             ;loop if more
;
;Pointer has relative host buffer address
sim_nooff       clc                     ;add hstbuf
                lda     #<hstbuf
                adc     sim_movepnt
                sta     sim_movepnt
                lda     #>hstbuf
                adc     sim_movepnt+1
                sta     sim_movepnt+1
;
;At this point pointer contains the address of the record of interest in the hstbuf buffer.
                ldy     #127            ;length of move - 1
                ldx     sim_readop      ;which way?
                bne     sim_rmove       ;skip if read
;
;Write operation so move from dmaadr to mvepnt
                inx                     ;x <-- 1
                stx     sim_hstwrt      ;hstwrt <-- 1
?L2             lda     (sim_dmaadr),y
                sta     (sim_movepnt),y
                dey
                bpl     ?L2             ;loop if more
                bmi     sim_endmve      ;else done
;
;Read operation so move from mvepnt to dmaadr
sim_rmove       lda     (sim_movepnt),y
                sta     (sim_dmaadr),y
                dey
                bpl     sim_rmove       ;loop if more
;
;Data has been moved to/from host buffer
sim_endmve      lda     sim_wrtype      ;write type
                cmp     #wrdir          ;to directory?
                bne     ?B2             ;done if not
;
;Clear host buffer for directory write
                lda     sim_erflag      ;get error flag
                bne     ?B2             ;done if errors
                sta     sim_hstwrt      ;say buffer written
                jsr     sim_writeh
?B2             lda     sim_erflag
                rts
;
;Writeh performs the physical write to the host disk.
sim_writeh      jsr     sim_hcom        ;set-up params
                jsr     mon_wrtsec      ;to rom
                sta     sim_erflag      ;save result
                rts
;
;Set parameters for host read/write
sim_hcom        lda     sim_hstdsk      ;set disk number
                sta     mon_nxtdrv      ;Save active Drive number
                lda     sim_hsttrk      ;then track
                sta     mon_nxttrk
                lda     sim_hstsec      ;then sector
                sta     mon_nxtsec
;
                lda     #<hstbuf        ;finally address
                ldy     #>hstbuf
                sta     lba_addr_low    ;Save low byte Buffer address
                sty     lba_addr_high   ;Save high byte Buffer address
                rts                     ;Return to Caller
;
;Set sector number
sim_selsec      sta     sim_seksec      ;save low and high
                sty     sim_seksec+1
                sta     mon_nxtsec
                rts
;
;Set buffer address
sim_setdma      sta     sim_dmaadr      ;store low
                sty     sim_dmaadr+1    ;and high
                ldx     #$01            ;set LBA count to 1
                jmp     B_IDE_SET_ADDR  ;use C02BIOS routine to finish
;
;Set track
sim_seltrk      sta     sim_sektrk      ;save number
                sty     sim_sektrk+1
                sta     mon_nxttrk
                rts
;
;**************************************************************************************************
;
;Console Input and Output Routines
; There are three routines for Console I/O
; 1- CONSTS - Console Status: tests to see if there is a character available
; 2- CONRDE - Console Read: waits for a character to be available, then returns with it
; 3- CONWRT - Console Write: sends a character to the console (preserves data)
;
mon_consts
                lda     icnt_a          ;Load Input Buffer count
                beq     conste          ;Branch if Buffer empty
                dec     a               ;Else, Decrement A (LDA #$FF)
conste          rts                     ;Return to Caller
;
mon_conrde
                jsr     b_chrin         ;Call BIOS for character in (waits)
                and     #$7F            ;Strip off high bit (ASCII only)
                cmp     #bs             ;Check for Backspace character
                bne     conrdx          ;If not, branch
                lda     #delete         ;Else, replace with Delete code
conrdx          rts                     ;Return to Caller
;
mon_conwrt
                pha                     ;Save Character to Stack
                and     #$7F            ;Strip off high bit (ASCII only)
                cmp     #delete         ;Check for Delete code
                bne     notdlt          ;If not, branch
                lda     #' '            ;Else, load ASCII space
                bra     ntceos          ;branch and send
notdlt          cmp     #clreol         ;Check for EOL code
                bne     ntceol          ;If not, branch
                lda     #esc            ;Else, load ASCII escape
                jsr     b_chrout        ;Call BIOS for character send
                lda     #'T'            ;Load upper case "T"
                bra     ntceos          ;Branch to send/exit
ntceol          cmp     #clreos         ;Check for CLR code
                bne     ntceos          ;If not, branch
                lda     #esc            ;Load ASCII escape
                jsr     b_chrout        ;Call BIOS for character send
                lda     #'Y'            ;Load upper case "Y"
ntceos          jsr     b_chrout        ;Call BIOS for character send
endvid          pla                     ;Get original Character from Stack
                rts                     ;return to Caller
;
;**************************************************************************************************
;
;Additional routines for using the second serial port via C02BIOS routines
;
mon_punch
                jmp     b_chrout2       ;send data to serial port 2
;
mon_reader
                jmp     b_chrin2        ;get data from serial port 2
;
;**************************************************************************************************
;
;Disk Routines for LBA Read and Write
; There are two routines here
; 1- RDESEC: Read an LBA into the Buffer
; 2- WRTSEC: Write an LBA from the Buffer
; These routines call the C02 BIOS for actual LBA read and write along with the routines
; to set the parameters for LBA requested and buffer address. These are 512-byte block accesses.
;
mon_wrtsec
                jsr     wrlba           ;Calculate CHS to LBA
                jsr     b_ide_write_lba ;Call BIOS to Write LBA
                bra     rd_wr_end       ;Branch to finish up
mon_rdesec
                jsr     wrlba           ;Calculate CHS to LBA
                jsr     b_ide_read_lba  ;Call BIOS to Read LBA
rd_wr_end
                lda     ide_status_ram  ;Get Status from BIOS call
                lsr     a               ;Shift error bit to carry
                bcs     ide_rw_err      ;Branch if error
                lda     #$00            ;Else, clear A reg
                rts                     ;Return to caller
ide_rw_err
                lda     #<io_err_msg    ;Point to error message
                ldy     #>io_err_msg
                jsr     m_promptr       ;Send it
                jmp     sim_boot        ;Restart SIM
;
;Calculate the LBA number from the Track and Sector parameters
; The Sector count can be from 0 to 255 (256 total)
; The Track count can be from 0 to 63 (64 total)
;
;24-bit addressing is supported by the C02BIOS V3.03
; The upper 4-bits (28-bit) are always zeroed by the BIOS routines,
; hence BIOS is limited to 8GB size as a result of 24-bit addressing.
;
;Once the Track and Sector values are moved to the LBA parameters,
; an offset must be added which is based on the Drive number (0-7).
; Partition sizes are limited to 8MB
; Number of drives is limited to 8 (A - H)
; Maximum available Data is limited to 64MB
;
;The Offset table is 4 bytes per entry and adds an LBA count to
; shift the absolute LBA numbers to the appropriate drive number
; being acessed (16384 blocks per).
;
; At entry;
; NXTSEC contains 0 to 255
;                       00000000|ssssssss
;
; NXTTRK contains 0 to 63
;                       00000000|0ttttttt
;
;First, copy NXTSEC to LBA0 with end result being:
;Second, copy NXTTRK to LBA1 and clear LBA2
;
; so LBA2|LBA1|LBA0 look like:
;                       00000000|0ttttttt|ssssssss
;
wrlba           lda     mon_nxtsec      ;Get requested Sector number
                sta     ide_lba0        ;Save to LBA lowest order byte
                lda     mon_nxttrk      ;Get requested Track number
                sta     ide_lba1        ;Save to LBA middle order byte
                stz     ide_lba2        ;Zero out the Upper order byte
;
;All LBA data starts as LBA 0 for each drive number.
; So the drive number is used as an index to the Offset table.
; This is then added to the 24-bit LBA address and stored to
; the BIOS variables for the requested LBA for Read or Write.
; Note: "LBA3" is not needed, as the BIOS handles 24-bit input
; and zeros the upper 4-bits of the (28-bit) address!
;
; Before adding the offset for the Drive number, we need to first add in
; the offset for the Partition we're running in. This is provided during
; the initial boot loader and seeded as a 32-bit long word.
;
; To manage this, we add in the partition offset to the the ide_lba(x) first,
; then select the drive via the routine below. This adds a small amount of
; processing overhead to each disk access but it's consistent and not that much.
; Again, this is only 24-bit effective addressing!
;
                clc                     ;Clear carry for add
                lda     ide_lba0        ;Get Low order LBA
                adc     part_offset+0   ;Add in low order offset
                sta     ide_lba0        ;Update Low order LBA
 
                lda     ide_lba1        ;Get Middle order LBA
                adc     part_offset+1   ;Add in high order offset
                sta     ide_lba1        ;Update Middle order LBA

                lda     ide_lba2        ;Get the High order LBA
                adc     part_offset+2   ;Add in extended order offset
                sta     ide_lba2        ;Update High order LBA
;
; The ide_lba(x) variables have been updated with the partition offset size
; Now all that's left is to add the specific offset for each drive letter
; then return and complete the block read/write operation. 

;
                lda     mon_nxtdrv      ;Get drive number
                asl     a               ;Multiply by two
                asl     a               ;Multiply by two (now = 4)
                tax                     ;Xfer offset to X reg
;
                clc                     ;Clear Carry for Add
                lda     ide_lba0        ;Get Low order LBA
                adc     offset,x        ;Add in low order offset
                sta     lba_low_byte    ;Save to BIOS variable
                inx                     ;Increment index to next offset byte
                lda     ide_lba1        ;Get Middle order LBA
                adc     offset,x        ;Add in high order offset
                sta     lba_high_byte   ;Save to BIOS variable
                inx                     ;Increment index to next offset byte
                lda     ide_lba2        ;Get the High order LBA
                adc     offset,x        ;Add in extended order offset
                sta     lba_ext_byte    ;Save to BIOS variable
                rts                     ;Return to Caller
;
;At this point LBA address is calculated and stored.
; The C02BIOS 3.03 variables are already loaded per the
; routine above. Using the LBA Read or Write function
; will transfer the LBA parameters, so we're done here.
;
;Table of LBA offsets for all drives A thru H (8MB each, 16KB Blocks)
; - NOTE: These offsets are based on a non-partitioned drive!!!
; - to provide a bootable drive with this code base, the boot routine
; - will need to add the partition offset to the values below!
; - This should be done during boot time before entering SIM, which starts the system.
;
offset
                .LONG   0               ;Drive A default
                .LONG   16384           ;Drive B
                .LONG   32768           ;Drive C
                .LONG   49152           ;Drive D
                .LONG   65536           ;Drive E
                .LONG   81920           ;Drive F
                .LONG   98304           ;Drive G
                .LONG   114688          ;Drive H
;Partition Offset
part_offset
                .LONG   0               ;Partition offset for drive letters
;
;**************************************************************************************************
;
;disk control blocks
; Updated to support 8 drives by default - KM
;drive a (CF 8MB)
dcb_a           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpa           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive b (CF 8MB)
dcb_b           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpb           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive c (CF 8MB)
dcb_c           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpc           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive d (CF 8MB)
dcb_d           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpd           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive e (CF 8MB)
dcb_e           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpe           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive f (CF 8MB)
dcb_f           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpf           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive g (CF 8MB)
dcb_g           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almpg           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;drive h (CF 8MB)
dcb_h           .DW     4095            ;max block number
                .DW     1024            ;records per track
                .DW     0               ;number system tracks
                .DB     1               ;block size = 2048
                .DW     1023            ;max directory
                .DW     almph           ;address of allocation map
                .DB     128             ;do not do checksums
                .DW     0               ;phony address
;
;**************************************************************************************************
;
; SIM Data area
;
sim_sekdsk      byte    0               ;seek disk number
sim_hstwrt      byte    0               ;0=written,1=pending host write
;allocate the following data areas to unused ram space
sim_hstact      defs    1               ;host active flag
sim_unacnt      defs    1               ;unalloc rec cnt
sim_sektrk      defs    2               ;seek track number
sim_seksec      defs    2               ;seek sector number
sim_hstdsk      defs    1               ;host disk number
sim_hsttrk      defs    2               ;host track number
sim_hstsec      defs    2               ;host sector number
sim_sekhst      defs    2               ;seek shr secshf
sim_unadsk      defs    1               ;last unalloc disk
sim_unatrk      defs    2               ;last unalloc track
sim_unasec      defs    2               ;last unalloc sector
sim_erflag      defs    1               ;error reporting
sim_rsflag      defs    1               ;read sector flag
sim_readop      defs    1               ;1 if read operation
sim_wrtype      defs    1               ;write operation type
sim_d65spt      defs    2               ;dos/65 records/track
;
;**************************************************************************************************
;
                end
